/*
 * Peony-Qt
 *
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yue Lan <lanyue@kylinos.cn>
 *
 */

#ifndef PRIMARYMANAGER_H
#define PRIMARYMANAGER_H

#include <QObject>
#include <QDBusConnection>
#include <QDBusMessage>
#include <QDBusInterface>

class PrimaryManager : public QObject
{
    Q_OBJECT
public:
    PrimaryManager();
    void start();
    int getScreenGeometry (QString methodName);
    QString getScreenName (QString methodName);

Q_SIGNALS:
    void priScreenChangedSignal(int x, int y, int width, int height);

public Q_SLOTS:
    void priScreenChanged(int x, int y, int width, int height);

private:
    QDBusInterface  *mDbusXrandInter;

};

#endif // PRIMARYMANAGER_H
